@@ -9,8 +9,8 @@
 # the mode switching program with the matching parameter
 # file from /usr/share/usb_modeswitch
 #
-# Part of usb-modeswitch-2.2.5 package
-# (C) Josua Dietze 2009-2015
+# Part of usb-modeswitch-2.4.0 package
+# (C) Josua Dietze 2009-2016
 
 set arg0 [lindex $argv 0]
 if [regexp {\.tcl$} $arg0] {
@@ -28,6 +28,8 @@ set flags(logging) 1
 set flags(noswitching) 0
 set flags(stordelay) 0
 set flags(logwrite) 0
+# also settable in device config files
+set flags(nombim) 0
 
 # Execution starts at file bottom
 
@@ -109,10 +111,7 @@ if {[string length [lindex $argList 0]] == 0} {
 		SafeExit
 	} else {
 		if {![regexp {(.*?):} [lindex $argList 1] d dev_top]} {
-			if [regexp {([0-9]+-[0-9]+\.?[0-9]*.*)} [lindex $argList 1] d dev_top] {
-				# new udev rules file, got to check class of first interface
-				set ifChk 1
-			} else {
+			if {![regexp {([0-9]+-[0-9]+\.?[0-9]*.*)} [lindex $argList 1] d dev_top]} {
 				Log "Could not determine device dir from udev values! Exit"
 				SafeExit
 			}
@@ -131,21 +130,15 @@ if {![file isdirectory $devdir]} {
 Log "Use top device dir $devdir"
 
 set iface 0
-if $ifChk {
-	Log "Check class of first interface ..."
-	set config(class) [IfClass 0]
-	if {$iface < 0} {
-		Log " No access to interface 0. Exit"
-		SafeExit
-	}
-	Log " Interface class is $config(class)."
-	if {$config(class) == "08" || $config(class) == "03"} {
-	} else {
-		Log "No install mode found. Aborting"
-		exit
-	}
+Log "Check class of first interface ..."
+set config(class) [IfClass 0 $devdir]
+if {$config(class) < 0} {
+	Log " No access to interface 0. Exit"
+	SafeExit
 }
-set ifdir [file tail [IfDir $iface]]
+Log " Interface 0 class is $config(class)."
+
+set ifdir [file tail [IfDir $iface $devdir]]
 regexp {:([0-9]+\.[0-9]+)$} $ifdir d iface
 
 set flags(logwrite) 1
@@ -250,7 +243,7 @@ foreach mconfig $configList {
 	Log "Check config: $mconfig"
 	if [MatchDevice $mconfig] {
 		Log "! matched. Read config data"
-		set flags(config) $mconfig
+#		set flags(config) $mconfig
 		if [string length $usb(busnum)] {
 			set busParam "-b [string trimleft $usb(busnum) 0]"
 			set devParam "-g [string trimleft $usb(devnum) 0]"
@@ -283,7 +276,6 @@ foreach mconfig $configList {
 					set cfgno [string trim $cfgno]
 					if {$cfgno > 0} {
 						set config(Configuration) $cfgno
-						set config(DriverModule) ""
 						set flags(config) "Configuration=$cfgno"
 					} else {
 						Log " No MBIM configuration found, switch to legacy modem mode"
@@ -298,6 +290,10 @@ foreach mconfig $configList {
 			set report "ok:busdev"
 			break
 		}
+		if {$config(Configuration) == 0} {
+			Log "Config file contains dummy method, do nothing. Exit"
+			SafeExit
+		}
 		UnbindDriver $devdir $ifdir
 		# Now we are actually switching
 		if $flags(logging) {
@@ -352,68 +348,19 @@ if [regexp {ok:busdev} $report] {
 	ReadUSBAttrs $devdir $ifdir
 }
 
-# Checking for bound drivers if there is an interface with class 0xff
+# driver binding removed !!
 
-if {$config(DriverModule) != "" && [regexp {ok:} $report]} {
-	if [HasFF] {
-		AddToList link_list $usb(idVendor):$usb(idProduct)
-	} else {
-		set config(DriverModule) ""
-		Log " No vendor-specific class found, skip driver check"
+if {[string length "$usb(idVendor)$usb(idProduct)"] < 8} {
+	if {![regexp {ok:(\w{4}):(\w{4})} $report d usb(idVendor) usb(idProduct)]} {
+		Log "No target vendor/product ID found or given, can't continue. Abort"
+		SafeExit
 	}
 }
-
-# If module is set (it is by default), driver shall be loaded.
-# If not, then NoDriverLoading is active
-
-if {$config(DriverModule) != ""} {
-	if {[string length "$usb(idVendor)$usb(idProduct)"] < 8} {
-		if {![regexp {ok:(\w{4}):(\w{4})} $report d usb(idVendor) usb(idProduct)]} {
-			Log "No target vendor/product ID found or given, can't continue. Abort"
-			SafeExit
-		}
-	}
-	# wait for any drivers to bind automatically
-	after 1500
-	Log "Now check for bound driver ..."
-	if {![file exists $devdir/$ifdir/driver]} {
-		Log " no driver has bound to interface 0 yet"
-
-		# If device is known, the sh wrapper will take care, else:
-		if {[InBindList $usb(idVendor):$usb(idProduct)] == 0} {
-			Log "Device is not in \"bind_list\" yet, bind it now"
-
-			# Load driver
-			CheckDriverBind $usb(idVendor) $usb(idProduct)
-
-			# Old/slow systems may take a while to create the devices
-			set counter 0
-			while {![file exists $devdir/$ifdir/driver]} {
-				if {$counter == 14} {break}
-				after 500
-				incr counter
-			}
-			if {$counter == 14} {
-				Log " driver binding failed"
-			} else {
-				Log " driver was bound to the device"
-				AddToList bind_list $usb(idVendor):$usb(idProduct)
-			}
-		}
-	} else {
-		Log " driver has bound, device is known"
-		if {[llength [glob -nocomplain $devdir/$ifdir/ttyUSB*]] > 0} {
-			AddToList link_list $usb(idVendor):$usb(idProduct)
-		}
-	}
-} else {
-	# Just in case "NoDriverLoading" was added after the first bind
-	RemoveFromBindList $usb(idVendor):$usb(idProduct)
-}
-
-if [regexp {ok:$} $report] {
-	# "NoDriverLoading" was set
-	Log "No driver check or bind for this device"
+# wait for drivers to bind
+after 500
+if {[llength [glob -nocomplain $devdir/$ifdir/ttyUSB*]] > 0} {
+	Log "Serial USB driver bound to interface 0\n will try to guess and symlink modem port on next connect"
+	AddToList link_list $usb(idVendor):$usb(idProduct)
 }
 
 # In newer kernels there is a switch to avoid the use of a device
@@ -606,8 +553,6 @@ return "Use global config file: $configFile"
 proc ParseDeviceConfig {cfg} {
 
 global config
-set config(DriverModule) ""
-set config(DriverIDPath) ""
 set config(WaitBefore) ""
 set config(TargetVendor) ""
 set config(TargetProduct) ""
@@ -616,7 +561,6 @@ set config(Configuration) ""
 set config(NoMBIMCheck) 0
 set config(PantechMode) 0
 set config(CheckSuccess) 20
-set loadDriver 1
 
 foreach pname [lsort [array names config]] {
 	if [regexp -line "^\[^# \]*?$pname.*?= *(0x(\\w+)|\"(\[0-9a-fA-F,\]+)\"|(\[0-9\]+)) *\$" $cfg d config($pname)] {
@@ -624,26 +568,6 @@ foreach pname [lsort [array names config]] {
 	}
 }
 
-if [regexp -line {^[^#]*?NoDriverLoading.*?=.*?(1|yes|true).*?$} $cfg] {
-	set loadDriver 0
-	Log "config: NoDriverLoading is set to active"
-}
-
-# For general driver loading; TODO: add respective device names.
-# Presently only useful for HSO devices (which are recounted now)
-if $loadDriver {
-	if {$config(DriverModule) == ""} {
-		set config(DriverModule) "option"
-		set config(DriverIDPath) "/sys/bus/usb-serial/drivers/option1"
-	} else {
-		if {$config(DriverIDPath) == ""} {
-			set config(DriverIDPath) "/sys/bus/usb/drivers/$config(DriverModule)"
-		}
-	}
-	Log "Driver module is \"$config(DriverModule)\", ID path is $config(DriverIDPath)\n"
-} else {
-	Log "Driver will not be handled by usb_modeswitch"
-}
 set config(WaitBefore) [string trimleft $config(WaitBefore) 0]
 
 }
@@ -724,7 +648,7 @@ if $flags(logwrite) {
 proc {SafeExit} {} {
 
 global flags
-set $flags(logwrite) 1
+set flags(logwrite) 1
 Log ""
 exit
 
@@ -844,92 +768,6 @@ return $symlinkName
 # end of proc {SymLinkName}
 
 
-# Load and bind driver (default "option")
-#
-proc {CheckDriverBind} {vid pid} {
-global config
-
-foreach fn {/sbin/modprobe /usr/sbin/modprobe} {
-	if [file exists $fn] {
-		set loader $fn
-	}
-}
-Log "Module loader is $loader"
-
-set idfile $config(DriverIDPath)/new_id
-if {![file exists $idfile]} {
-	if {$loader == ""} {
-		Log "Can't do anymore without module loader; get \"modtools\"!"
-		return
-	}
-	Log "\nTry to load module \"$config(DriverModule)\""
-	if [catch {set result [exec $loader -v $config(DriverModule)]} err] {
-		Log " Running \"$loader $config(DriverModule)\" gave an error:\n  $err"
-	} else {
-		Log " Module was loaded successfully:\n$result"
-	}
-} else {
-	Log "Module is active already"
-}
-set i 0
-while {$i < 50} {
-	if [file exists $idfile] {
-		break
-	}
-	after 20
-	incr i
-}
-if {$i < 50} {
-	Log "Try to add ID to driver \"$config(DriverModule)\""
-	SysLog "usb_modeswitch: add device ID $vid:$pid to driver \"$config(DriverModule)\""
-	SysLog "usb_modeswitch: please report the device ID to the Linux USB developers!"
-	if [catch {exec echo "$vid $pid ff" >$idfile} err] {
-		Log " Error adding ID to driver:\n  $err"
-	} else {
-		Log " ID added to driver; check for new devices in /dev"
-	}
-} else {
-	Log " \"$idfile\" not found, check if kernel version is at least 2.6.27"
-	Log "Fall back to \"usbserial\""
-	set config(DriverModule) usbserial
-	Log "\nTry to unload driver \"usbserial\""
-	if [catch {exec $loader -r usbserial} err] {
-		Log " Running \"$loader -r usbserial\" gave an error:\n  $err"
-		Log "No more fallbacks"
-		return
-	}
-	after 50
-	Log "\nTry to load driver \"usbserial\" with device IDs"
-	if [catch {set result [exec $loader -v usbserial vendor=0x$vid product=0x$pid]} err] {
-		Log " Running \"$loader usbserial\" gave an error:\n  $err"
-	} else {
-		Log " Driver was loaded successfully:\n$result"
-	}
-}
-
-}
-# end of proc {CheckDriverBind}
-
-
-# Check if USB ID is listed as needing driver binding
-proc {InBindList} {id} {
-
-set listfile /var/lib/usb_modeswitch/bind_list
-if {![file exists $listfile]} {return 0}
-set rc [open $listfile r]
-set buffer [read $rc]
-close $rc
-if [string match *$id* $buffer] {
-Log "Found $id in bind_list"
-	return 1
-} else {
-Log "No $id in bind_list"
-	return 0
-}
-
-}
-# end of proc {InBindList}
-
 # Add USB ID to list of devices needing later treatment
 proc {AddToList} {name id} {
 
@@ -953,42 +791,15 @@ close $lc
 # end of proc {AddToList}
 
 
-# Remove USB ID from bind list (NoDriverLoading is set)
-proc {RemoveFromBindList} {id} {
-
-set listfile /var/lib/usb_modeswitch/bind_list
-if [file exists $listfile] {
-	set rc [open $listfile r]
-	set buffer [read $rc]
-	close $rc
-	set idList [split [string trim $buffer] \n]
-} else {
-	return
-}
-set idx [lsearch $idList $id]
-if {$idx > -1} {
-	set idList [lreplace $idList $idx $idx]
-} else {
-	return
-}
-if {[llength $idList] == 0} {
-	file delete $listfile
-	return
-}
-set buffer [join $idList "\n"]
-if [catch {set lc [open $listfile w]}] {return}
-puts $lc $buffer
-close $lc
-
-}
-# end of proc {RemoveFromBindList}
-
-
 proc {CheckSuccess} {devdir} {
 
 global config usb flags
-set ifdir [file tail [IfDir 0]]
 
+# For Cisco AM10, target device not on same port
+if {$usb(idVendor) == "1307" && $usb(idProduct) == "1169"} {
+	set devdir [string range $devdir 0 end-1]2
+}
+set ifdir [file tail [IfDir 0 $devdir]]
 if {[string length $config(TargetClass)] || [string length $config(Configuration)]} {
 	set config(TargetVendor) $usb(idVendor)
 	set config(TargetProduct) $usb(idProduct)
@@ -1004,7 +815,7 @@ for {set i 1} {$i <= $config(CheckSuccess)} {incr i} {
 	} else {
 		Log " Read attributes ..."
 	}
-	set ifdir [IfDir 0]
+	set ifdir [IfDir 0 $devdir]
 	if {$ifdir == ""} {continue}
 	set ifdir [file tail $ifdir]
 	if {![ReadUSBAttrs $devdir $ifdir]} {
@@ -1050,9 +861,8 @@ if {$i > 20} {return 0} else {return 1}
 # end of proc {CheckSuccess}
 
 
-proc {IfDir} {iface} {
+proc {IfDir} {iface devdir} {
 
-global devdir
 set allfiles [glob -nocomplain $devdir/*]
 set files [glob -nocomplain $devdir/*.$iface]
 if {[llength $files] == 0} {
@@ -1067,9 +877,9 @@ return $ifdir
 }
 # end of proc {IfDir}
 
-proc {IfClass} {iface} {
+proc {IfClass} {iface devdir} {
 
-set ifdir [IfDir $iface]
+set ifdir [IfDir $iface $devdir]
 
 if {![file exists $ifdir/bInterfaceClass]} {
 	return -1
@@ -1188,10 +998,10 @@ if $flags(logging) {
 
 }
 
-proc {HasFF} {} {
+proc {HasFF} {devdir} {
 
 set i 0
-while {[set dir [IfDir $i]] != ""} {
+while {[set dir [IfDir $i $devdir]] != ""} {
 	set c [exec cat $dir/bInterfaceClass]
 	if {$c == "ff"} {return 1}
 	incr i
